@@ -16,17 +16,18 @@
 import argparse
 import json
 import logging
+import platform
 import subprocess
 import sys
 
 from pathlib import Path
-from typing import Optional, Sequence
+from typing import List, Optional, Sequence
 
 from mako.exceptions import RichTraceback
 from tqdm import tqdm
 
 from .api_reference import ApiReference
-from .generator import process_adoc, AsciiDocError
+from .generator import process_adoc
 from .model import json_repr
 from .packaging import CollectError, PackageManager, SpecificationError
 from .parser.doxygen import Driver as DoxygenDriver
@@ -41,14 +42,27 @@ def error(*args, **kwargs) -> None:
 
 def asciidoctor(destination_dir: Path, out_file: Path, processed_file: Path, multipage: bool,
                 backend: str, extra_args: Sequence[str], image_dir: Path) -> None:
-    subprocess.run([
-        f"asciidoctor -D {destination_dir} -o {out_file} -b {backend} "
-        f"{'-a multipage ' if multipage else ''}"
-        f"-a imagesdir@={image_dir} "
-        f"{processed_file} {' '.join(extra_args)}"
-    ],
-                   shell=True,
-                   check=True)
+    args = [
+        "asciidoctor",
+        "-D",
+        str(destination_dir),
+        "-o",
+        str(out_file),
+        "-b",
+        backend,
+        "-a",
+        f"imagesdir@={image_dir}",
+        str(processed_file),
+    ]
+    if multipage:
+        args += ["-a", "multipage"]
+    if extra_args:
+        args += extra_args
+
+    if platform.system() == "Windows":
+        subprocess.run(args, shell=True, check=True)
+    else:
+        subprocess.run(" ".join(args), shell=True, check=True)
 
 
 def output_extension(backend: str) -> Optional[str]:
@@ -225,13 +239,8 @@ def main(argv: Optional[Sequence[str]] = None) -> None:
                                               multipage=args.multipage,
                                               progress=progress)
 
-    except AsciiDocError as e:
-        logger.error(f"Error while processing AsciiDoc file:\n\t{e}")
-        logger.error("\nTraceback:")
-        traceback = RichTraceback()
-        for filename, lineno, _, line in traceback.traceback:
-            if filename.endswith(".adoc"):
-                logger.error(f"File {filename}, line {lineno}:\n\t{line}\n")
+    except:  # noqa: E722
+        logger.error(human_traceback(pkg_mgr))
         sys.exit(1)
 
     in_dir = in_file.parent
@@ -250,5 +259,46 @@ def main(argv: Optional[Sequence[str]] = None) -> None:
             pkg_mgr.make_image_directory(destination_dir, progress)
 
 
+def human_traceback(pkg_mgr: PackageManager) -> str:
+    """Generate a human readable traceback the current exception. To be used inside an except
+    clause.
+
+    Exceptions triggered inside AsciiDoc are handled specially:
+     - Lines inside AsciiDoc files are resolved to the original AsciiDoc lines.
+     - AsciiDoxy code loading the AsciiDoc files is skipped.
+     - Additional Mako loading due to includes is skipped.
+    """
+    traceback = RichTraceback()
+
+    message = [""]
+    has_adoc = False
+    pending_traceback: List[str] = []
+    for filename, lineno, function, line in traceback.traceback:
+        if filename.endswith(".adoc"):
+            pending_traceback.clear()
+
+            package_name, original_file = pkg_mgr.find_original_file(Path(filename))
+            if package_name and package_name != "INPUT":
+                filename = f"{package_name}:/{original_file}"
+            elif original_file is not None:
+                filename = str(original_file)
+            message.append(f"  File {filename}, line {lineno}, in AsciiDoc\n    {line}")
+            has_adoc = True
+        elif "/mako/" in filename:
+            continue
+        else:
+            pending_traceback.append(f"  File {filename}, line {lineno}, in "
+                                     f"{function}\n    {line}")
+    if pending_traceback:
+        message.extend(pending_traceback)
+
+    if has_adoc:
+        message[0] = f"Error while processing AsciiDoc files:\n{traceback.error}\nTraceback:"
+    else:
+        message[0] = f"Internal error:\n{traceback.error}\nTraceback:"
+
+    return "\n".join(message)
+
+
 if __name__ == "__main__":
     main()
