@@ -1,4 +1,4 @@
-# Copyright (C) 2019-2020, TomTom (http://tomtom.com).
+# Copyright (C) 2019-2021, TomTom (http://tomtom.com).
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,8 +22,7 @@ from typing import Callable, Mapping, Optional, Tuple, Type, TypeVar, Union
 
 from ..api_reference import ApiReference
 from ..generator.errors import AsciiDocError
-from ..model import (Compound, EnumValue, InnerTypeReference, Parameter, ReferableElement,
-                     ReturnValue, ThrowsClause, TypeRef, TypeRefBase)
+from ..model import Compound, Parameter, ReferableElement, ReturnValue, ThrowsClause, TypeRef
 
 
 class TranscoderError(AsciiDocError):
@@ -111,10 +110,6 @@ class TranscoderBase(ABC):
         transcoded = self.referable_element(compound)
 
         transcoded.members = [self.compound(m) for m in compound.members]
-        transcoded.inner_classes = [
-            self.inner_type_reference(itr) for itr in compound.inner_classes
-        ]
-        transcoded.enumvalues = [self.enum_value(ev) for ev in compound.enumvalues]
         transcoded.params = [self.parameter(p) for p in compound.params]
         transcoded.exceptions = [self.throws_clause(tc) for tc in compound.exceptions]
         transcoded.returns = (self.return_value(compound.returns)
@@ -139,7 +134,11 @@ class TranscoderBase(ABC):
         return transcoded
 
     def type_ref(self, type_ref: TypeRef) -> TypeRef:
-        transcoded = self.type_ref_base(type_ref)
+        transcoded = TypeRef(self.TARGET)
+
+        transcoded.id = self.convert_id(type_ref.id)
+        transcoded.name = self.convert_name(type_ref)
+        transcoded.namespace = type_ref.namespace
 
         transcoded.kind = self.convert_kind(type_ref)
         transcoded.prefix = type_ref.prefix
@@ -179,27 +178,6 @@ class TranscoderBase(ABC):
 
         return transcoded
 
-    def enum_value(self, enum_value: EnumValue) -> EnumValue:
-        return self.find_or_transcode(enum_value, self._enum_value)
-
-    def _enum_value(self, enum_value: EnumValue) -> EnumValue:
-        transcoded = self.referable_element(enum_value)
-
-        transcoded.initializer = enum_value.initializer
-        transcoded.brief = enum_value.brief
-        transcoded.description = enum_value.description
-
-        return transcoded
-
-    def inner_type_reference(self, ref: InnerTypeReference) -> InnerTypeReference:
-        transcoded = self.type_ref_base(ref)
-
-        transcoded.prot = ref.prot
-        if ref.referred_object is not None:
-            transcoded.referred_object = self.compound(ref.referred_object)
-
-        return transcoded
-
     def convert_id(self, old_id: Optional[str]) -> Optional[str]:
         if not old_id:
             return old_id
@@ -212,7 +190,7 @@ class TranscoderBase(ABC):
     def convert_kind(self, source_element: Union[ReferableElement, TypeRef]) -> Optional[str]:
         return source_element.kind
 
-    def convert_name(self, source_element: Union[ReferableElement, TypeRefBase]) -> str:
+    def convert_name(self, source_element: Union[ReferableElement, TypeRef]) -> str:
         return source_element.name
 
     def convert_full_name(self, source_element: ReferableElement) -> str:
@@ -244,14 +222,3 @@ class TranscoderBase(ABC):
         transcoded.kind = self.convert_kind(element) or ""
 
         return transcoded
-
-    RefType = TypeVar("RefType", bound=TypeRefBase)
-
-    def type_ref_base(self, type_ref: RefType) -> RefType:
-        transcoded = type_ref.__class__(self.TARGET)
-
-        transcoded.id = self.convert_id(type_ref.id)
-        transcoded.name = self.convert_name(type_ref)
-        transcoded.namespace = type_ref.namespace
-
-        return transcoded
